import enum


class Counties(enum.Enum):
    USTRIA = 'at_AT'
    BELGIUM = 'fr_BE'
    CZECH_REPUBLIC = 'cz_CZ'
    GERMANY = 'de_DE'
    DENMARK = 'da_DK'
    SPAIN = 'es_ES'
    FINLAND = 'fi_FI'
    FRANCE = 'fr_FR'
    HONG_KONG = 'en_HK'
    CROATIA = 'hr_HR'
    HUNGARY = 'hu_HU'
    INDONESIA = 'id_ID'
    ISRAIL = 'he_IL'
    IRAN = 'fa_IR'
    ITALY = 'it_IT'
    JAPAN = 'ja_JP'
    SOUTH_KOREA = 'ko_KR'
    LITHUANIA = 'lt_LT'
    LATVIA = 'lv_lv'
    MALAYSIA = 'ms_MY'
    NIGERIA = 'en_NG'
    NETHERLANDS = 'nl_NL'
    NEPAL = 'ne_NP'
    NEW_ZEALAND = 'en_NZ'
    PERU = 'es_PE'
    SAUDI_ARABIA = 'ar_SA'
    USA = 'en_US'
    VENEZUELA = 'es_VE'
    SOUTH_AFRICA = 'en_ZA'



from ..receive import country
from .phone import Phone
from ..exceptions import ProblemWithFetchNumbers
from bs4 import BeautifulSoup
import re


class Country(country.Country):
    def __init__(self, s, _country, url):
        super().__init__(s, _country, url)

    def get_numbers(self):
        if self._phones:
            return self._phones

        def __parse(page_number):
            r = self._s.get(self._url + f'{page_number}.html')
            if r.status_code != 200:
                raise ProblemWithFetchNumbers()
            page = BeautifulSoup(r.text, 'html.parser')
            try:
                _is_need_break = not page.find('ul', {'class': 'pagination'}).find_all('li')[-1].find('a').get('href', False)
            except AttributeError:
                _is_need_break = True

            return [Phone(self._s, self._country, e.find('a')['href'], re.findall(r'.*/(\d*)/', e.find('a')['href'])[0], page.find('h1').text.split()[-1]) for e in page.find_all('li', {'class': 'wow'})], \
                _is_need_break

        i = 0
        while True:
            i += 1
            data, is_need_break = __parse(i)
            self._phones += data
            if is_need_break:
                break
        return self._phones



class Letter:
    """
    Base class of letter
    """
    def __init__(self, email, name, from_email, subject, send_time, proxies):
        self._email = email
        self._name = name
        self._from_email = from_email
        self._subject = subject
        self._send_time = send_time
        self._proxies = proxies
        self._letter = None
        self._letter_id = None

    @property
    def email(self):
        """
        :return: Email address to which the letter was sent.
        """
        return self._email

    @property
    def name(self):
        """
        :return: Name of the person who sent the letter.
        """
        return self._name

    @property
    def from_email(self):
        """
        :return: Email address from which the email was sent.
        """
        return self._from_email

    @property
    def subject(self):
        """
        :return: Subject of letter.
        """
        return self._subject

    @property
    def send_time(self):
        """
        :return: Datetime when letter sent.
        """
        return self._send_time

    @property
    def letter(self):
        """
        :return: Content of letter.
        """
        return ''

    def __repr__(self):
        return '<Letter name={} from_email={} email={} subject={} send_time={}>'.format(self._name, self._from_email,
                                                                                        self._email, self._subject,
                                                                                        self._send_time.strftime('%Y-%m-%d %H:%M:%S'))

    def __hash__(self):
        return hash(self._letter_id)



import enum


class Domains(enum.Enum):
    GMAILNATOR_COM = ['{}@gmailnator.com']
    PSNATOR_COM = ['{}@psnator.com']
    GMAIL_COM = ['claytmp+{}@gmail.com',
                 'ednatmp+{}@gmail.com',
                 'tinytmp+{}@gmail.com',
                 'irontmp+{}@gmail.com',
                 'combtmp+{}@gmail.com',
                 'dueltmp+{}@gmail.com',
                 'aveltmp+{}@gmail.com',
                 'sisatmp+{}@gmail.com',
                 'wipetmp+{}@gmail.com',
                 'jocktmp+{}@gmail.com',
                 'daratmp+{}@gmail.com',
                 'joneltmp+{}@gmail.com',
                 'hcarrieyt+{}@gmail.com',
                 'hjefferyt+{}@gmail.com']



from account_generator_helper.utilities import str_to_timedelta
from datetime import datetime


class Message:
    def __init__(self, sender_name, message, string_time):
        self._sender_name = sender_name
        self._message = message
        self._send_time = datetime.now() - str_to_timedelta(string_time)

    @property
    def sender_name(self):
        """
        :return: Name of sender name.
        """
        return self._sender_name

    @property
    def message(self):
        """
        :return: Message content.
        """
        return self._message

    @property
    def send_time(self):
        """
        :return: Time when message sent.
        """
        return self._send_time

    def __repr__(self):
        return '<Message from={} message={} send_time={}>'.format(self._sender_name, self._message,
                                                                  self._send_time.strftime('%Y-%m-%d %H:%M:%S'))



import random
import re
import string
from urllib.parse import quote
from datetime import timedelta

try:
    from html import unescape
except ImportError:
    from html.parser import HTMLParser
    unescape = HTMLParser().unescape

LETTERS = string.ascii_letters + '1234567890'


def random_string():
    return ''.join([random.choice(LETTERS) for _ in range(12)])


def str_to_timedelta(string_time):
    _time = int(string_time.split()[0])
    if 'second' in string_time:
        return timedelta(seconds=_time)
    elif 'min' in string_time:
        return timedelta(minutes=_time)
    elif 'hour' in string_time:
        return timedelta(hours=_time)
    elif 'day' in string_time:
        return timedelta(days=_time)
    elif 'month' in string_time:
        return timedelta(days=_time*30)
    elif 'year' in string_time:
        return timedelta(days=_time*365)
    return timedelta()


def camel_to_snake(text):
    return re.sub(r'(?<!^)(?=[A-Z])', '_', text).lower()



import random
import requests
from ..exceptions import ProblemWithGetEmail
from ..mail import Mail
from .domains import Domains
from account_generator_helper.utilities import random_string, quote
from .letter import Letter

headers = {
  'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
}


class GmailNator(Mail):
    """
    Class for work with https://www.gmailnator.com/
    """
    def __init__(self, proxy=None):
        super().__init__(proxy)
        self.__s = requests.Session()
        if proxy:
            self.__s.proxies.update(self._proxies)
        self.__s.get('https://www.gmailnator.com/')

    def __get_csrf_gmail_nator_token(self):
        return self.__s.cookies.get('csrf_gmailnator_cookie')

    def __get_payload(self, action, additional_data=''):

        return f'csrf_gmailnator_token={self.__get_csrf_gmail_nator_token()}&action={action}' + additional_data

    def get_email(self):
        return self.set_email(random_string(), Domains[random.choice(Domains._member_names_)])

    def get_email_online(self, use_custom_domain=True, use_plus=True, use_point=True):
        """
        Random email address from the site.

        :param use_custom_domain: Generate email not only from @gmail.com.
        :param use_plus: Generate email with "+" in address.
        :param use_point:  Generate email with "." in address.
        :return: Random email address.
        """

        payload = '&'.join(['', *[f"data[]={i+1}" for i, data in enumerate(list(locals().values())[1:]) if data]])
        r = self.__s.post('https://www.gmailnator.com/index/indexquery', headers=headers, data=self.__get_payload('GenerateEmail', payload))
        if r.status_code == 200:
            return self._set_email(r.json()['email'])
        raise ProblemWithGetEmail()

    def set_email(self, email, domain: Domains = Domains.GMAILNATOR_COM):
        return self._set_email(random.choice(domain.value).format(email))

    def get_inbox(self):
        payload = self.__get_payload('LoadMailList', f'&Email_address={quote(self._email)}')
        r = self.__s.post('https://www.gmailnator.com/mailbox/mailboxquery', headers=headers, data=payload)
        if r.status_code == 200:
            return [Letter(self._email, _letter['content'], self._proxies) for _letter in r.json() if 'inbox' in _letter['content']]
        return []



import re
import time
from account_generator_helper.utilities import random_string


class Mail:
    """
    Base class of mail service
    """
    def __init__(self, proxy=None):
        """
        :param proxy: (optional) Proxy string.
        """
        self._email = None
        self._proxies = {'http': proxy, 'https': proxy} if proxy else None
        self._handlers = []

    def get_email(self, *args, **kwargs):
        """
        Generates a random address and returns it.

        :return: Random email address.
        """
        return self.set_email(random_string())

    def set_email(self, *args, **kwargs):
        pass

    def _set_email(self, mail):
        self._email = mail
        return self._email

    def get_inbox(self):
        """
        Return all letters from inbox.

        :return: List of Letter objects.
        """
        pass

    def letter_handler(self, name='', from_email='', subject='', re_subject=''):
        """
        Letter handler decorator.
        This decorator can be used to decorate functions that must handle certain types of letters.
        All letters handlers are tested in the order they were added.

        :param name: (Optional) Name of the person who sent the letter.
        :param from_email: (Optional) Email address from which the letter was sent.
        :param subject: (Optional) Letter subject.
        :param re_subject: (Optional) The same as subject but use regular expression.
        """
        def wrapper(handler):
            self._handlers.append({'handler': handler, 'name': name, 'from_email': from_email,
                                   'subject': subject, 're_subject': re_subject})

        return wrapper

    def _letter_handler(self, _letter):
        def __is_valid(handler):
            return (not handler['name'] or handler['name'] == _letter.name) and \
                   (not handler['from_email'] or handler['from_email'] == _letter.from_email) and \
                   (not handler['subject'] or handler['subject'] == _letter.subject) and \
                   (not handler['re_subject'] or re.findall(handler['re_subject'], _letter.subject))

        for _handler in self._handlers:
            if __is_valid(_handler):
                _handler['handler'](_letter)

    def poling(self, timeout: int = 10):
        """
        Must use with letter_handler:

        :param timeout: (Optional) Timeout between inbox update, default 10 seconds.
        """
        letters = [hash(_letter) for _letter in self.get_inbox()]
        try:
            while True:
                time.sleep(timeout)
                for _letter in self.get_inbox():
                    if hash(_letter) not in letters:
                        letters.append(hash(_letter))
                        self._letter_handler(_letter)
        except KeyboardInterrupt:
            pass



import enum


class Domains(enum.Enum):
    MAILTO_PLUS = 'mailto.plus'
    FEXPOST_COM = 'fexpost.com'
    FEXBOX_ORG = 'fexbox.org'
    FEXBOX_RU = 'fexbox.ru'
    MAILBOX_IN_UA = 'mailbox.in.ua'
    ROVER_INFO = 'rover.info'
    INPWA_COM = 'inpwa.com'
    INTOPWA_COM = 'intopwa.com'
    TOFEAT_COM = 'tofeat.com'
    CHITTHI_IN = 'chitthi.in'


from setuptools import setup
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name='account_generator_helper',
    version='1.0.2',
    packages=['account_generator_helper', 'account_generator_helper.fake_data', 'account_generator_helper.temp_mail',
              'account_generator_helper.temp_mail.mail', 'account_generator_helper.temp_mail.gmailnator',
              'account_generator_helper.temp_mail.inboxkitten', 'account_generator_helper.temp_mail.tempmailplus',
              'account_generator_helper.temp_phone', 'account_generator_helper.temp_phone.receive',
              'account_generator_helper.temp_phone.receive_sms'],
    url='https://github.com/DioniS1902/AccountGeneratorHelper',
    license='',
    author='Dionis1902',
    author_email='den70007.ua@gmail.com',
    description='This library is helpful when creating accounts, it has everything you need for this',
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=required
)
